//
//	T2090CGTestView.swift
//	T2020-09-CG
//
//	Created by Kaz Yoshikawa on 9/17/20.
//

import UIKit

public protocol ZArchivable {
	func archive() -> Data
}

public extension ZArchivable {
	func archive() -> Data {
		var mutableValue: Self = self
		let unsafeMutableRawPointer = UnsafeMutableRawPointer(&mutableValue)
		return NSData(bytes: unsafeMutableRawPointer, length: MemoryLayout<Self>.size) as Data
	}
}

public extension Data {
	func unarchive<T: ZArchivable>() -> T? {
		guard self.count >= MemoryLayout<T>.size else { print("too small"); return nil }
		let unsafeRawPointer = (self as NSData).bytes
		let unsafePointer = UnsafePointer<T>(OpaquePointer(unsafeRawPointer))
		return unsafePointer.pointee
	}
}

struct StrokePoint: ZArchivable {
	var point: CGPoint
	var duration: UInt16
	var majorRadius: Float16
	var force: Float16
	var altitudeAngle: Float16
	var azimuthAngle: Float16
}

struct Stroke {
	var points: [CGPoint]
	var width: CGFloat
	var opacity: CGFloat
}

class T2009CGTestView: UIView {
	
	func loadStrokes(named name: String) -> [Stroke] {
		let url = Bundle.main.url(forResource: name, withExtension: "plist")!
		let dictionary = NSDictionary(contentsOf: url)!
		var strokes = [Stroke]()
		for layer in  dictionary["layers"] as! [NSDictionary] {
			for drawables in layer["drawables"] as! [NSDictionary] where drawables["type"] as? String == "stroke" {
				let opacity = drawables["opacity"] as! CGFloat
				let width = drawables["width"] as! CGFloat
				var points = [CGPoint]()
				for data in drawables["points"] as! [Data] {
					if let point = data.unarchive() as StrokePoint? {
						points.append(point.point)
					}
				}
				let stroke = Stroke(points: points, width: width, opacity: opacity)
				strokes.append(stroke)
			}
		}
		return strokes
	}

	lazy var strokes: [Stroke] = {
		return self.loadStrokes(named: "9D972943-7613-4AFD-A261-7CD617EE543B")
//		return self.loadStrokes(named: "183E3AE3-D105-4519-BF05-2E5C131C9F52")
	}()


	override func draw(_ rect: CGRect) {
		let context = UIGraphicsGetCurrentContext()!
		context.saveGState()
		defer { context.restoreGState() }

		context.translateBy(x: 940, y: 640)
//		context.scaleBy(x: 0.75, y: 0.75)

		for stroke in self.strokes {
			context.setStrokeColor(UIColor.black.withAlphaComponent(0.125).cgColor)
			context.setLineWidth(stroke.width)
			context.setLineJoin(.round)
			context.setLineCap(.round)
//			context.interpolationQuality = .high
			for (index, point) in stroke.points.enumerated() {
				let pt = CGPoint(x: CGFloat(point.x), y: CGFloat(point.y))
				if index == 0 {
					context.move(to: pt)
				}
				else {
					context.addLine(to: pt)
				}
			}
		}
		context.strokePath()
	}
	
}
